@@ -12,9 +12,9 @@
 ##############################################################################
 """ Basic ZPublisher request management.
 
-$Id: BaseRequest.py 70357 2006-09-24 17:59:04Z regebro $
+$Id: BaseRequest.py 105081 2009-10-15 11:17:39Z witsch $
 """
-from urllib import quote
+from urllib import quote as urllib_quote
 import xmlrpc
 from zExceptions import Forbidden, Unauthorized, NotFound
 from Acquisition import aq_base
@@ -35,6 +35,10 @@ from zope.traversing.namespace import nsParse, namespaceLookup
 
 UNSPECIFIED_ROLES=''
 
+def quote(text):
+    # quote url path segments, but leave + and @ intact
+    return urllib_quote(text, '/+@')
+
 try:
     from ExtensionClass import Base
     class RequestContainer(Base):
@@ -117,7 +121,10 @@ class DefaultPublishTraverse(object):
                     pass
 
                 # Lastly we try with key access:
-                subobject = object[name]
+                try:
+                    subobject = object[name]
+                except TypeError: # unsubscriptable
+                    raise KeyError(name)
                 
 
         # Ensure that the object has a docstring, or that the parent
@@ -196,10 +203,15 @@ class BaseRequest:
         else: other.update(kw)
         self.other=other
 
-    def close(self):
+    def clear(self):
         self.other.clear()
         self._held=None
+
+    def close(self):
         notify(EndRequestEvent(None, self))
+        # subscribers might need the zodb, so `clear` must come afterwards
+        # (since `self._held=None` might close the connection, see above)
+        self.clear()
 
     def processInputs(self):
         """Do any input processing that could raise errors
@@ -427,6 +439,17 @@ class BaseRequest:
                     # BrowserDefault returns the object to be published
                     # (usually self) and a sequence of names to traverse to
                     # find the method to be published.
+                    
+                    # This is webdav support. The last object in the path
+                    # should not be acquired. Instead, a NullResource should
+                    # be given if it doesn't exist:
+                    if (no_acquire_flag and
+                        hasattr(object, 'aq_base') and 
+                        not hasattr(object,'__bobo_traverse__')):
+                        if object.aq_parent is not object.aq_inner.aq_parent:
+                            from webdav.NullResource import NullResource
+                            object = NullResource(parents[-2], object.getId(), self).__of__(parents[-2])
+                    
                     if IBrowserPublisher.providedBy(object):
                         adapter = object
                     else:
@@ -619,7 +642,8 @@ class BaseRequest:
     def _hold(self, object):
         """Hold a reference to an object to delay it's destruction until mine
         """
-        self._held=self._held+(object,)
+        if self._held is not None:
+            self._held=self._held+(object,)
 
 def exec_callables(callables):
     result = None
