@@ -13,7 +13,7 @@
 
 __version__='$Revision: 1.96 $'[11:-2]
 
-import re, sys, os, time, random, codecs
+import re, sys, os, time, random, codecs, inspect
 from types import StringType, UnicodeType
 from BaseRequest import BaseRequest, quote
 from HTTPResponse import HTTPResponse
@@ -25,6 +25,8 @@ from TaintedString import TaintedString
 from maybe_lock import allocate_lock
 xmlrpc=None # Placeholder for module that we'll import if we have to.
 
+from zope.publisher.base import DebugFlags
+
 # This may get overwritten during configuration
 default_encoding = 'iso-8859-15'
 
@@ -136,7 +138,7 @@ class HTTPRequest(BaseRequest):
         r.retry_count=self.retry_count
         return r
 
-    def close(self):
+    def clear(self):
         # Clear all references to the input stream, possibly
         # removing tempfiles.
         self.stdin = None
@@ -146,7 +148,7 @@ class HTTPRequest(BaseRequest):
         # one.  Without this, there's the possibility of memory leaking
         # after every request.
         self._lazies = {}
-        BaseRequest.close(self)
+        BaseRequest.clear(self)
 
     def setServerURL(self, protocol=None, hostname=None, port=None):
         """ Set the parts of generated URLs. """
@@ -262,6 +264,7 @@ class HTTPRequest(BaseRequest):
         self.steps=[]
         self._steps=[]
         self._lazies={}
+        self._debug = DebugFlags()
 
 
         if environ.has_key('REMOTE_ADDR'):
@@ -269,7 +272,15 @@ class HTTPRequest(BaseRequest):
             if environ.has_key('HTTP_X_FORWARDED_FOR') and self._client_addr in trusted_proxies:
                 # REMOTE_ADDR is one of our trusted local proxies. Not really very remote at all.
                 # The proxy can tell us the IP of the real remote client in the forwarded-for header
-                self._client_addr = environ['HTTP_X_FORWARDED_FOR'].split(',')[-1].strip()
+                # Skip the proxy-address itself though
+                forwarded_for = [
+                    e.strip()
+                    for e in environ['HTTP_X_FORWARDED_FOR'].split(',')]
+                forwarded_for.reverse()
+                for entry in forwarded_for:
+                    if entry not in trusted_proxies:
+                        self._client_addr = entry
+                        break
         else:
             self._client_addr = ''
 
@@ -1038,7 +1049,7 @@ class HTTPRequest(BaseRequest):
         try: object=req.traverse(path)
         except: rsp.exception()
         if object is None:
-            req.close()
+            req.clear()
             raise rsp.errmsg, sys.exc_info()[1]
 
         # The traversal machinery may return a "default object"
@@ -1056,7 +1067,7 @@ class HTTPRequest(BaseRequest):
         if name != os.path.split(path)[-1]:
             object=req.PARENTS[0]
 
-        req.close()
+        req.clear()
         return object
 
 
@@ -1213,7 +1224,18 @@ class HTTPRequest(BaseRequest):
             raise KeyError, key
         return v
 
+    # Using the getattr protocol to retrieve form values and similar
+    # is discouraged and is likely to be deprecated in the future.
+    # request.get(key) or request[key] should be used instead
     def __getattr__(self, key, default=_marker, returnTaints=0):
+        # ugly hack to make request.debug work for Zope 3 code (the
+        # ZPT engine, to be exact) while retaining request.debug
+        # functionality for all other code
+        if key == 'debug':
+            lastframe = inspect.currentframe().f_back
+            if lastframe.f_globals['__name__'].startswith('zope.'):
+                return self._debug
+        
         v = self.get(key, default, returnTaints=returnTaints)
         if v is _marker:
             raise AttributeError, key
@@ -1449,7 +1471,7 @@ def parse_cookie(text,
                  qparmre=re.compile(
                     '([\x00- ]*([^\x00- ;,="]+)="([^"]*)"([\x00- ]*[;,])?[\x00- ]*)'),
                  parmre=re.compile(
-                    '([\x00- ]*([^\x00- ;,="]+)=([^\x00- ;,"]*)([\x00- ]*[;,])?[\x00- ]*)'),
+                    '([\x00- ]*([^\x00- ;,="]+)=([^;,"]*)([\x00- ]*[;,])?[\x00- ]*)'),
                  paramlessre=re.compile(
                     '([\x00- ]*([^\x00- ;,="]+)[\x00- ]*[;,][\x00- ]*)'),
 
@@ -1484,11 +1506,11 @@ def parse_cookie(text,
 
             else:
                 # Broken Cookie without = nor value.
- 		broken_p = paramlessre.match(text)
- 		if broken_p:
- 		    l = len(broken_p.group(1))
- 		    name = broken_p.group(2)
- 		    value = ''
+                broken_p = paramlessre.match(text)
+                if broken_p:
+                    l = len(broken_p.group(1))
+                    name = broken_p.group(2)
+                    value = ''
 
                 else:
                     return result
@@ -1507,7 +1529,7 @@ class record:
     _guarded_writes = 1
 
     def __getattr__(self, key, default=None):
-        if key in ('get', 'keys', 'items', 'values', 'copy', 'has_key'):
+        if key in ('get', 'keys', 'items', 'values', 'copy', 'has_key', '__contains__', '__iter__', '__len__'):
             return getattr(self.__dict__, key)
         raise AttributeError, key
 
